#include "mooapp.h"
#include "mainwindow.h"
#include "connectionmanager.h"
#include "connection.h"
#include "listener.h"
#include "taskentry.h"
#include <QApplication>
#include <lua.hpp>
#include <QDateTime>
#include <QFile>
#include "objectmanager.h"

MainWindow		*gMainWindow = 0;

void myMessageOutput( QtMsgType type, const char *msg )
{
	QFile				LogFil( "moo.log" );
	const QDateTime		CurDat = QDateTime::currentDateTime();
	QString				DateTime = CurDat.toString( "dd-MMM-yyyy hh:mm:ss" );
	QString				LogMsg;

	switch( type )
	{
		default:
			LogMsg = QString( "%1 - %2" ).arg( DateTime ).arg( msg );
	}

	if( gMainWindow != 0 )
	{
		gMainWindow->log( LogMsg );
	}

	if( LogFil.open( QIODevice::Append | QIODevice::Text ) )
	{
		QTextStream		TxtOut( &LogFil );

		TxtOut << LogMsg << "\n";

		LogFil.close();
	}
}

int main( int argc, char *argv[] )
{
	QApplication	a( argc, argv );

	a.setApplicationName( "ArtMOO" );
	a.setOrganizationName( "Alex May" );
	a.setOrganizationDomain( "http://www.bigfug.com" );

	MainWindow		w;

	gMainWindow = &w;

	qInstallMsgHandler( myMessageOutput );

	w.show();

	qDebug() << "ArtMOO v0.1.0 by Alex May - www.bigfug.com";
	qDebug() << QString( "%1 %2" ).arg( LUA_RELEASE ).arg( LUA_COPYRIGHT );

	QString			DataFileName = "moo.dat";

	const QStringList	args = a.arguments();

	foreach( const QString &arg, args )
	{
		if( arg.startsWith( "-db=", Qt::CaseSensitive ) )
		{
			DataFileName = arg.mid( 4 );
		}
	}

	mooApp			 m( DataFileName );

	//ObjectManager	&OM = *ObjectManager::instance();

	//w.installModel( &OM );

	ConnectionManager	&CM = *ConnectionManager::instance();

	Listener			 l( 0 );

	CM.doConnect( 0 );

	int		r = a.exec();

	qDebug() << "ArtMOO exiting\n";

	return( r );
}
