#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QDebug>
#include <QWindowStateChangeEvent>

MainWindow::MainWindow( QWidget *pParent )
	: QMainWindow( pParent ), ui( new Ui::MainWindow )
{
	ui->setupUi( this );

	setWindowIcon( QIcon( ":/moo-icon.ico" ) );

	mTrayIcon = new QSystemTrayIcon( QIcon( ":/moo-icon.ico" ), this );

	connect( mTrayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(trayActivated(QSystemTrayIcon::ActivationReason)));

	mTrayIcon->setToolTip( "ArtMOO" );

	mTrayIcon->show();

}

MainWindow::~MainWindow()
{
	delete ui;
}

void MainWindow::log(const QString &pMessage)
{
	ui->plainTextEdit->appendPlainText( pMessage );
}

void MainWindow::installModel( QAbstractItemModel *pModel )
{
	ui->treeView->setModel( pModel );
}

void MainWindow::trayActivated( QSystemTrayIcon::ActivationReason pReason )
{
	if( pReason == QSystemTrayIcon::Trigger )
	{
		if( isMinimized() )
		{
			setParent( NULL, Qt::Window );

			showNormal();
		}
	}
}

bool MainWindow::event( QEvent *pEvent )
{
	if( pEvent->type() == QEvent::WindowStateChange )
	{
		if( isMinimized() )
		{
			setParent( NULL, Qt::SubWindow );

			pEvent->ignore();
		}
		else
		{
			pEvent->accept();
		}
	}

	return( QMainWindow::event( pEvent ) );
}
